      SUBROUTINE DDTRMV(UPLO,TRANS,DIAG,N,AH,AL,LDA,XH,XL,INCX)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      INTEGER INCX,LDA,N
      CHARACTER DIAG,TRANS,UPLO
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),AL(LDA,*),XH(*),XL(*)
*     ..
*
*  Purpose
*  =======
*
*  DDTRMV  performs one of the matrix-vector operations
*
*     x := A*x,   or   x := A^T*x,
*
*  where x is an n element vector and  A is an n by n unit, or non-unit,
*  upper or lower triangular matrix.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the matrix is an upper or
*           lower triangular matrix as follows:
*
*              UPLO = 'U' or 'u'   A is an upper triangular matrix.
*
*              UPLO = 'L' or 'l'   A is a lower triangular matrix.
*
*           Unchanged on exit.
*
*  TRANS  - CHARACTER*1.
*           On entry, TRANS specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'   x := A*x.
*
*              TRANS = 'T' or 't'   x := A^T*x.
*
*              TRANS = 'C' or 'c'   x := A^T*x.
*
*           Unchanged on exit.
*
*  DIAG   - CHARACTER*1.
*           On entry, DIAG specifies whether or not A is unit
*           triangular as follows:
*
*              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
*
*              DIAG = 'N' or 'n'   A is not assumed to be unit
*                                  triangular.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AH must contain the high 
*           word of upper triangular matrix and the strictly lower 
*           triangular part of AH is not referenced.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AH must contain the high
*           word of lower triangular matrix and the strictly upper 
*           triangular part of AH is not referenced.
*           Note that when  DIAG = 'U' or 'u', the diagonal elements of
*           AH are not referenced either, but are assumed to be unity.
*           Unchanged on exit.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AL must contain the low 
*           word of upper triangular matrix and the strictly lower 
*           triangular part of AL is not referenced.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AL must contain the low
*           word of lower triangular matrix and the strictly upper 
*           triangular part of AL is not referenced.
*           Note that when  DIAG = 'U' or 'u', the diagonal elements of
*           AL are not referenced either, but are assumed to be unity.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, n ).
*           Unchanged on exit.
*
*  XH     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XH must contain the n
*           high word of element vector x. On exit, XH is overwritten
*           with the tranformed vector XH.
*  XL     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XL must contain the n
*           low word of element vector x. On exit, XL is overwritten
*           with the tranformed vector XL.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  Further Details
*  ===============
*
*  Level 2 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      DOUBLE PRECISION T1H,T1L,T2H,T2L
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      INTEGER I,INFO,IX,J,JX,KX
      LOGICAL NOUNIT
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDTRMV"
#endif
*
*     Test the input parameters.
*
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (.NOT.LSAME(TRANS,'N') .AND. .NOT.LSAME(TRANS,'T') .AND.
     +         .NOT.LSAME(TRANS,'C')) THEN
          INFO = 2
      ELSE IF (.NOT.LSAME(DIAG,'U') .AND. .NOT.LSAME(DIAG,'N')) THEN
          INFO = 3
      ELSE IF (N.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,N)) THEN
          INFO = 6
      ELSE IF (INCX.EQ.0) THEN
          INFO = 8
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDTRMV',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
      IF (N.EQ.0) RETURN
*
      NOUNIT = LSAME(DIAG,'N')
*
*     Set up the start point in X if the increment is not unity. This
*     will be  ( N - 1 )*INCX  too small for descending loops.
*
      IF (INCX.LE.0) THEN
          KX = 1 - (N-1)*INCX
      ELSE IF (INCX.NE.1) THEN
          KX = 1
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  x := A*x.
*
          IF (LSAME(UPLO,'U')) THEN
              IF (INCX.EQ.1) THEN
                  DO 20 J = 1,N
!                     IF (X(J).NE.ZERO) THEN
                      IF (XH(J).NE.ZERO.OR.XL(J).NE.ZERO) THEN
!                         TEMP = X(J)
                          T1H = XH(J)
                          T1L = XL(J)
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                          DO 10 I = 1,J - 1
!                             X(I) = X(I) + TEMP*A(I,J)
!                      call dd_mul(T2H,T2L,T1H,T1L,AH(I,J),AL(I,J))
          T2H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_selfadd(XH(I),XL(I),T2H,T2L)
          p1 = XH(I) + T2H
          p2 = p1 - XH(I)
          q2 = (XH(I) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + XL(I)
          q2 = q2 + T2L
          XH(I) = q1 + q2
          XL(I) = q2 - (XH(I) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   10                     CONTINUE
!$omp end do
!$omp end parallel
!                         IF (NOUNIT) X(J) = X(J)*A(J,J)
                          IF (NOUNIT) THEN
!                     call dd_selfmul(XH(J),XL(J),AH(J,J),AL(J,J))
          q1 = XH(J)*AH(J,J)
          u1 = 134217729.0*XH(J)
          p1 = u1 - (u1-XH(J))
          p2 = XH(J) - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * XH(J)
          q2 = q2 + AH(J,J) * XL(J)
          XH(J) = q1 + q2
          XL(J) = q2 - (XH(J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                          ENDIF
                      END IF
   20             CONTINUE
              ELSE
!                  JX = KX
                  DO 40 J = 1,N
                  JX = KX+INCX*(J-1)
!                     IF (X(JX).NE.ZERO) THEN
                      IF (XH(JX).NE.ZERO.OR.XL(JX).NE.ZERO) THEN
!                         TEMP = X(JX)
                          T1H = XH(JX)
                          T1L = XL(JX)
!                          IX = KX
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
!$omp& private(IX)
                          DO 30 I = 1,J - 1
                          IX = KX+INCX*(I-1)
!                             X(IX) = X(IX) + TEMP*A(I,J)
!                     call dd_mul(T2H,T2L,T1H,T1L,AH(I,J),AL(I,J))
          T2H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_selfadd(XH(IX),XL(IX),T2H,T2L)
          p1 = XH(IX) + T2H
          p2 = p1 - XH(IX)
          q2 = (XH(IX) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + XL(IX)
          q2 = q2 + T2L
          XH(IX) = q1 + q2
          XL(IX) = q2 - (XH(IX) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                              IX = IX + INCX
   30                     CONTINUE
!$omp end do
!$omp end parallel
!                         IF (NOUNIT) X(JX) = X(JX)*A(J,J)
                          IF (NOUNIT) THEN
!                     call dd_selfmul(XH(JX),XL(JX),AH(J,J),AL(J,J))
          q1 = XH(JX)*AH(J,J)
          u1 = 134217729.0*XH(JX)
          p1 = u1 - (u1-XH(JX))
          p2 = XH(JX) - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * XH(JX)
          q2 = q2 + AH(J,J) * XL(JX)
          XH(JX) = q1 + q2
          XL(JX) = q2 - (XH(JX) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                          ENDIF
                      END IF
!                      JX = JX + INCX
   40             CONTINUE
              END IF
          ELSE
              IF (INCX.EQ.1) THEN
                  DO 60 J = N,1,-1
!                     IF (X(J).NE.ZERO) THEN
                      IF (XH(J).NE.ZERO.OR.XL(J).NE.ZERO) THEN
!                         TEMP = X(J)
                          T1H = XH(J)
                          T1L = XL(J)
!$omp  parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                          DO 50 I = N,J + 1,-1
!                             X(I) = X(I) + TEMP*A(I,J)
!                     call dd_mul(T2H,T2L,T1H,T1L,AH(I,J),AL(I,J))
          T2H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_selfadd(XH(I),XL(I),T2H,T2L)
          p1 = XH(I) + T2H
          p2 = p1 - XH(I)
          q2 = (XH(I) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + XL(I)
          q2 = q2 + T2L
          XH(I) = q1 + q2
          XL(I) = q2 - (XH(I) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50                     CONTINUE
!$omp end do
!$omp end parallel
!                         IF (NOUNIT) X(J) = X(J)*A(J,J)
                          IF (NOUNIT) THEN
!                     call dd_selfmul(XH(J),XL(J),AH(J,J),AL(J,J))
          q1 = XH(J)*AH(J,J)
          u1 = 134217729.0*XH(J)
          p1 = u1 - (u1-XH(J))
          p2 = XH(J) - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * XH(J)
          q2 = q2 + AH(J,J) * XL(J)
          XH(J) = q1 + q2
          XL(J) = q2 - (XH(J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                          ENDIF
                      END IF
   60             CONTINUE
              ELSE
                  KX = KX + (N-1)*INCX
!                  JX = KX
                  DO 80 J = N,1,-1
                  JX= KX - INCX*(N-J)
!                     IF (X(JX).NE.ZERO) THEN
                      IF (XH(JX).NE.ZERO.OR.XL(JX).NE.ZERO) THEN
!                         TEMP = X(JX)
                          T1H = XH(JX)
                          T1L = XL(JX)
                          IX = KX
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
!$omp& private(IX)
                          DO 70 I = N,J + 1,-1
                          IX = KX - INCX*(N-I)
!                             X(IX) = X(IX) + TEMP*A(I,J)
!                     call dd_mul(T2H,T2L,T1H,T1L,AH(I,J),AL(I,J))
          T2H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     call dd_selfadd(XH(IX),XL(IX),T2H,T2L)
          p1 = XH(IX) + T2H
          p2 = p1 - XH(IX)
          q2 = (XH(IX) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + XL(IX)
          q2 = q2 + T2L
          XH(IX) = q1 + q2
          XL(IX) = q2 - (XH(IX) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                              IX = IX - INCX
   70                     CONTINUE
!$omp end do
!$omp end parallel
!                         IF (NOUNIT) X(JX) = X(JX)*A(J,J)
                          IF (NOUNIT) THEN
!                     call dd_selfmul(XH(JX),XL(JX),AH(J,J),AL(J,J))
          q1 = XH(JX)*AH(J,J)
          u1 = 134217729.0*XH(JX)
          p1 = u1 - (u1-XH(JX))
          p2 = XH(JX) - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * XH(JX)
          q2 = q2 + AH(J,J) * XL(JX)
          XH(JX) = q1 + q2
          XL(JX) = q2 - (XH(JX) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                          ENDIF
                      END IF
!                      JX = JX - INCX
   80             CONTINUE
              END IF
          END IF
      ELSE
*
*        Form  x := A^T*x.
*
          IF (LSAME(UPLO,'U')) THEN
              IF (INCX.EQ.1) THEN
                  DO 100 J = N,1,-1
!                     TEMP = X(J)
                      T1H = XH(J)
                      T1L = XL(J)
!                     IF (NOUNIT) TEMP = TEMP*A(J,J)
                      IF (NOUNIT) THEN
!                     call dd_selfmul(T1H,T1L,AH(J,J),AL(J,J))
          q1 = T1H*AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * T1H
          q2 = q2 + AH(J,J) * T1L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                      ENDIF
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                      DO 90 I = J - 1,1,-1
!                         TEMP = TEMP + A(I,J)*X(I)
!                     call dd_mul(T2H,T2L,AH(I,J),AL(I,J),XH(I),XL(I))
          T2H = AH(I,J) * XH(I)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(I)
          r1 = u1 - (u1-XH(I))
          r2 = XH(I) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (AH(I,J) * XL(I) + AL(I,J) * XH(I))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp critical (name0)
!                     call dd_selfadd(T1H,T1L,T2H,T2L)
          p1 = T1H + T2H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T2L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   90                 CONTINUE
!$omp end do
!$omp end parallel
!                     X(J) = TEMP
                      XH(J) = T1H
                      XL(J) = T1L
  100             CONTINUE
              ELSE
!                  JX = KX + (N-1)*INCX
                  DO 120 J = N,1,-1
                  JX= KX + (N-1)*INCX - (N-J)*INCX
!                     TEMP = X(JX)
                      T1H = XH(JX)
                      T1L = XL(JX)
!                      IX = JX
!                     IF (NOUNIT) TEMP = TEMP*A(J,J)
                      IF (NOUNIT) THEN
!                     call dd_selfmul(T1H,T1L,AH(J,J),AL(J,J))
          q1 = T1H*AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * T1H
          q2 = q2 + AH(J,J) * T1L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                      ENDIF
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
!$omp& private(IX)
                      DO 110 I = J - 1,1,-1
!                          IX = IX - INCX
                          IX = JX - (J-I)*INCX
!                         TEMP = TEMP + A(I,J)*X(IX)
!                   call dd_mul(T2H,T2L,AH(I,J),AL(I,J),XH(IX),XL(IX))
          T2H = AH(I,J) * XH(IX)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(IX)
          r1 = u1 - (u1-XH(IX))
          r2 = XH(IX) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (AH(I,J) * XL(IX) + AL(I,J) * XH(IX))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp critical (name0)
!                   call dd_selfadd(T1H,T1L,T2H,T2L)
          p1 = T1H + T2H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T2L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  110                 CONTINUE
!$omp end do
!$omp end parallel
!                     X(JX) = TEMP
                      XH(JX) = T1H
                      XL(JX) = T1L
!                      JX = JX - INCX
  120             CONTINUE
              END IF
          ELSE
              IF (INCX.EQ.1) THEN
                  DO 140 J = 1,N
!                     TEMP = X(J)
                      T1H = XH(J)
                      T1L = XL(J)
!                     IF (NOUNIT) TEMP = TEMP*A(J,J)
                      IF (NOUNIT) THEN
!                     call dd_selfmul(T1H,T1L,AH(J,J),AL(J,J))
          q1 = T1H*AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * T1H
          q2 = q2 + AH(J,J) * T1L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                      ENDIF
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                      DO 130 I = J + 1,N
!                         TEMP = TEMP + A(I,J)*X(I)
!                     call dd_mul(T2H,T2L,AH(I,J),AL(I,J),XH(I),XL(I))
          T2H = AH(I,J) * XH(I)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(I)
          r1 = u1 - (u1-XH(I))
          r2 = XH(I) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (AH(I,J) * XL(I) + AL(I,J) * XH(I))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp critical (name0)
!                     call dd_selfadd(T1H,T1L,T2H,T2L)
          p1 = T1H + T2H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T2L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  130                 CONTINUE
!$omp end do
!$omp end parallel
!                     X(J) = TEMP
                      XH(J) = T1H
                      XL(J) = T1L
  140             CONTINUE
              ELSE
!                  JX = KX
                  DO 160 J = 1,N
                  JX = KX + INCX*(J-1)
!                     TEMP = X(JX)
                      T1H = XH(JX)
                      T1L = XL(JX)
!                      IX = JX
!                     IF (NOUNIT) TEMP = TEMP*A(J,J)
                      IF (NOUNIT) THEN
!                     call dd_selfmul(T1H,T1L,AH(J,J),AL(J,J))
          q1 = T1H*AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + AL(J,J) * T1H
          q2 = q2 + AH(J,J) * T1L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
                      ENDIF
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
!$omp& private(IX)
                      DO 150 I = J + 1,N
!                          IX = IX + INCX
                          IX = JX + INCX*(I-J)
!                         TEMP = TEMP + A(I,J)*X(IX)
!                   call dd_mul(T2H,T2L,AH(I,J),AL(I,J),XH(IX),XL(IX))
          T2H = AH(I,J) * XH(IX)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(IX)
          r1 = u1 - (u1-XH(IX))
          r2 = XH(IX) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (AH(I,J) * XL(IX) + AL(I,J) * XH(IX))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp critical (name0)
!                   call dd_selfadd(T1H,T1L,T2H,T2L)
          p1 = T1H + T2H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T2L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  150                 CONTINUE
!$omp end do
!$omp end parallel
!                     X(JX) = TEMP
                      XH(JX) = T1H
                      XL(JX) = T1L
                      JX = JX + INCX
  160             CONTINUE
              END IF
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDTRMV"
      WRITE(6,'(2A)') "UPLO      = ",UPLO
      WRITE(6,'(2A)') "TRANS     = ",TRANS
      WRITE(6,'(2A)') "DIAG      = ",DIAG
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDTRMV .
*
      END
