package Test::QPBLAS;

use strict;
use warnings;

use Carp;
use List::Util qw(min max);
use Test::More;

my $THRESHOLD = 1e-10;
my $INFINITY = 'inf' + 0; 

sub get_threshold {
  return $THRESHOLD;
}

sub set_threshold {
  $THRESHOLD = shift;
}

sub supply_default_dimensions {
  return (1, 2, 9);
}

sub supply_default_increments {
  return (1, 2, -1, -2);
}

sub supply_default_alphabeta {
  return (-1, 0, 1);
}

sub supply_default_side {
  return ('l', 'r');
}

sub supply_default_uplo {
  return ('u', 'l');
}

sub supply_default_diag {
  return ('n', 'u');
}

sub supply_default_trans {
  return ('n', 't', 'c');
}

my $TEMP_IN =  "$$.in.tmp";
my $TEMP_OUT = "$$.out.tmp";

sub run {
  my($routine_name, $conditions) = @_;
  local $\ = "\n"; #say

  $routine_name = uc $routine_name;
  my($level, $routine_id) = get_routine($routine_name);

  open my $condition_file, '>:utf8', $TEMP_IN or die $!;
  for my $keyword (keys %$conditions) {
    print $condition_file get_keyword($keyword);
    print $condition_file $conditions->{$keyword};
  }
  print $condition_file get_keyword('VERIFY');
  print $condition_file $routine_id;
  print $condition_file get_keyword('EXIT');
  close $condition_file;

  my $description = get_description($routine_name, $conditions);
  my $pid = fork;
  if (!$pid) {
    open my $out, '>:utf8', $TEMP_OUT or die $!;
    print $out $description;
    close $out;
    local $SIG{ALRM} = sub { exit };
    alarm 5;
    exec "./test/level$level <$TEMP_IN >>$TEMP_OUT";
  }
  else {
    wait;
  }

  open my $result_file, '<:utf8', $TEMP_OUT or die $!;
  my %results = ( d => [], dd => [] );
  while (<$result_file>) {
    if (/(\w+ Malloc Error)/) {
      fail $description;
      diag $1;
      return;
    }
    next unless /$routine_name/;
    if (/NaN/) {
      fail $description;
      diag "NaN detected";
      return;
    }
    if (/\[(I?)[QW]$routine_name G?\1([DZ]{1,2})$routine_name .+:\s*([-+.E0-9*]+)/) {
      my($is_dd, $error) = (length $2 == 2, $3);
      $error =~ s/(\d)([-+]\d)/$1E$2/;
      $error = $INFINITY if $error =~ /\*/;
      push @{$results{$is_dd ? 'dd' : 'd'}}, $error;
    }
  }
  close $result_file;

  my $should_be_quad = min @{$results{dd}};
  my $should_be_double = max @{$results{d}};
  if (!defined $should_be_quad || !defined $should_be_double) {
    fail $description;
  }

  my $ratio = $should_be_double == 0.0 ? $should_be_quad : $should_be_quad / $should_be_double;
  cmp_ok($ratio, '<', $THRESHOLD, $description)
    || diag("double: $should_be_double, quad: $should_be_quad");

  if ($should_be_double > $THRESHOLD) {
    diag "double is too large";
  }
}

END {
  unlink $TEMP_IN;
  unlink $TEMP_OUT;
}


my %ROUTINES = (
  SWAP  => [1,  1],  SCAL  => [1,  2],  COPY  => [1,  3],  AXPY  => [1,  4],
  DOT   => [1,  5],  NRM2  => [1,  6],  ASUM  => [1,  7],  AMAX  => [1,  8],
  ROT   => [1,  9],  ROTG  => [1, 10],  ROTM  => [1, 11],  ROTMG => [1, 12],
  DOTC  => [1, 13],  DOTU  => [1, 14],
  GEMV  => [2,  1],  SYMV  => [2,  2],  TRMV  => [2,  3],  TRSV  => [2,  4],
  SYR   => [2,  5],  SYR2  => [2,  6],  GBMV  => [2,  7],  GER   => [2,  8],
  SBMV  => [2,  9],  TBMV  => [2, 10],  TBSV  => [2, 11],  GERC  => [2, 12],
  GERU  => [2, 13],  HBMV  => [2, 14],  HEMV  => [2, 15],  HER   => [2, 16],
  HER2  => [2, 17],
  GEMM  => [3,  1],  SYMM  => [3,  2],  SYR2K => [3,  3],  SYRK  => [3,  4],
  TRMM  => [3,  5],  TRSM  => [3,  6],  HEMM  => [3,  7],  HER2K => [3,  8],
  HERK  => [3,  9],
);

my %KEYWORDS = (
  INCX => 70, INCY => 71, ALPHA => 72, BETA => 73,
  M => 74, N => 75, K => 76, KL => 76, KU => 77,
  UPLO => 80, DIAG => 81, SIDE => 82, TRANS => 83, TRANSA => 83, TRANSB => 84,
  MULTI => 96, VERBOSE => 97, VERIFY => 98, BENCHMARK => 99, EXIT => 0,
);

sub get_routine {
  my $routine_name = shift;
  if (!exists $ROUTINES{uc $routine_name}) {
    croak "unknown routine: $routine_name";
  }
  return @{$ROUTINES{uc $routine_name}};
}

sub get_keyword {
  my $keyword = shift;
  if (!exists $KEYWORDS{uc $keyword}) {
    croak "unknown keyword: $keyword";
  }
  return $KEYWORDS{uc $keyword};
}

sub get_description {
  my($routine_name, $condition) = @_;
  my $str = $routine_name. ": ";
  $str .= join ', ', ( map { "$_=$condition->{$_}" } keys %$condition);
  return $str;
}


1;
